﻿/*
  Script:	gd_lib.js
  Gadget:	N/A
  Purpose:	General purpose support library for gadgets, common functions here
  Author:	Ben Coleman
  Update:	08 Oct 2007
  Notes:	Tested on Google Desktop v5.1
  Changes:	1.0.0.0	- Initial release
  Changes:	1.0.1.0	- Fixed issue with potential empty HTTP response

*/

//
// General purpose AJAX object for making HTTP calls
//  Parameters: call_back_handler = name of a function that will be called with the HTTP response
//  		pass_text = boolean; does the call back function get called with the HTTP response Text or XMLDOM
//		err_handler = optional name of a function that is called in the event of an error
//
// The call_back_handler function should have a single parameter, which when called will contain either the HTTP
// response text or the XML DOM object (depending on the value of pass_text) the default is to pass the XML DOM
// The err_handler function should have a single parameter, which when called will contain the 
// unerlying XMLHttpRequest object
//
function AJAX(call_back_handler, pass_text, err_handler)
{
	this.http 		= new XMLHttpRequest();

	this.call_back 		= call_back_handler;
	this.xml_or_text 	= pass_text;
	this.error_call_back 	= err_handler;
	// set this to true for debuging and logging info
	this.log 		= false;

	this.user;
	this.pwd;

	this.sendRequest = function(url)
	{
		if(this.log) debug.trace("Sending request to "+url);
		try {
			this.http.open("GET", url, true, this.user, this.pwd);

			// Force cacheing off, fix for IE, coz it's crap
			ifModifiedSince = new Date(0); // January 1, 1970
			this.http.setRequestHeader("Pragma", "no-cache");
			this.http.setRequestHeader("Cache-Control", "no-cache");
			this.http.setRequestHeader("If-Modified-Since", ifModifiedSince);
			
			this.http.send();

			this.http.onreadystatechange = createObjectCallback(this, this.httpResponse);
			
			return true;
		} catch (e) {
			if(this.log) debug.error("Call to: "+url+" failed "+e);
			return false;
		}			
	}

	//
	// Can be used to set a username and password used by the HTTP requests
	//
	this.setAuth = function(u, p)
	{
		this.user = u;
		this.pwd = p;
	}

	//
	// Internal callback used when the XMLHttpRequest has done it's stuff
	// used to chain the call to the user specified call back function
	//
	this.httpResponse = function()
	{
		if(this.http.readyState == 4) {			
			if(this.log) debug.trace("Got response: "+this.http.status);
			if(this.http.status == 200) {
				if(this.xml_or_text) {
					if(this.log) debug.trace("Invoking callback with text content");
					txt = "";
					if(typeof(this.http.responseText) != "unknown")
						txt = this.http.responseText;
					this.call_back(txt);
				} else { 
					if(this.log) debug.trace("Invoking callback with XML DOM");
					this.call_back(this.http.responseXML);	
				}
			} else {
				if(this.error_call_back) this.error_call_back(this.http);	
			}
		}			
	}
}


//
// helper function to augment the standard Array object with an indexOf function
//
Array.prototype.indexOf = function(v)
{
	for(ix = 0; ix < this.length; ix++) {
		if(this[ix] == v) return ix;
	}
	return -1;
}

//
// helper function to augment the standard String object with a trim function
//
String.prototype.trim = function()
{
	return this.replace(/^\s+|\s+$/g, "");
}


//
// madness - special voodoo to allow pseodo OO callbacks to work... found on the interweb
//
function createObjectCallback(obj, fn)
{
	return function() { fn.apply(obj, arguments); };
}

//
// lazy printing to stdout
//
function sout(m)
{
	debug.trace(""+m);
}

//
// decrypt the HTTP codes returned by XMLHttpRequest 
//  - These errors come from wininet, the underlying network implementation used by XMLHttpRequest on Windows 
//
function lookupHTTPErrorCode(code){
	if(code == 12002) return "The request has timed out";
	if(code == 12007) return "The server name could not be resolved";
	if(code == 12029) return "The attempt to connect to the server failed";
	return "Unknown error";
}

//
//  General purpose serialisation function to serialize an Object into a string of name value pairs
//  Delimiter is ASCII code HEX 03 (ETX)
//
function serializeObject(obj)
{
	var arr = new Array();
	for(var prop in obj) {
		if(typeof(obj[prop]) != "function") {
			arr.push(prop);
			arr.push(obj[prop]);
		}
	}
	return arr.join("\x03");
}

//
//  General purpose serialisation function to serialize an Array of Objects into a string
//  Delimiter is ASCII code HEX 04 (EOT)
//
function serializeObjectArray(arr)
{
	var temp = new Array();
	for(var i = 0; i < arr.length; i++) {
		temp.push(serializeObject(arr[i]));
	}
	return temp.join("\x04");
}

//
// De-serialize an object from a string created by serializeObject, back into a javascript Object 
// second parameter should be the object class name as a string e.g. "Employee" will create Employee objects
//
function deserializeObject(str, obj_type)
{
	eval("var obj = new "+obj_type+"();");

	var str_parts = str.split("\x03");
	for(var i = 0; i < str_parts.length; i+=2) {
		//sout(str_parts[i]+"==="+str_parts[i+1]);
		obj[str_parts[i]] = str_parts[i+1];
	}
	return obj;
}

//
// De-seralize an Array of Objects from a string created by serializeObjectArray
// second parameter is the same as for deserializeObject
//
function deserializeObjectArray(str, obj_type)
{
	var temp = new Array();
	if(str.length == 0) return temp;
	var str_parts = str.split("\x04");

	for(var i = 0; i < str_parts.length; i++) {
		var o = deserializeObject(str_parts[i], obj_type);
		temp.push(o);
	}
	return temp;
}

//
// field validation functions
// all are nicked from the javascript toolbox website
//

//-------------------------------------------------------------------
// isNull(value)
//   Returns true if value is null
//-------------------------------------------------------------------
function isNull(val){return(val==null);}

//-------------------------------------------------------------------
// isBlank(value)
//   Returns true if value only contains spaces
//-------------------------------------------------------------------
function isBlank(val){
	if(val==null){return true;}
	for(var i=0;i<val.length;i++) {
		if ((val.charAt(i)!=' ')&&(val.charAt(i)!="\t")&&(val.charAt(i)!="\n")&&(val.charAt(i)!="\r")){return false;}
		}
	return true;
	}

//-------------------------------------------------------------------
// isInteger(value)
//   Returns true if value contains all digits
//-------------------------------------------------------------------
function isInteger(val){
	if (isBlank(val)){return false;}
	for(var i=0;i<val.length;i++){
		if(!isDigit(val.charAt(i))){return false;}
		}
	return true;
	}

//-------------------------------------------------------------------
// isNumeric(value)
//   Returns true if value contains a positive float value
//-------------------------------------------------------------------
function isNumeric(val){return(parseFloat(val,10)==(val*1));}

//-------------------------------------------------------------------
// isArray(obj)
// Returns true if the object is an array, else false
//-------------------------------------------------------------------
function isArray(obj){return(typeof(obj.length)=="undefined")?false:true;}

//-------------------------------------------------------------------
// isDigit(value)
//   Returns true if value is a 1-character digit
//-------------------------------------------------------------------
function isDigit(num) {
	if (num.length>1){return false;}
	var string="1234567890";
	if (string.indexOf(num)!=-1){return true;}
	return false;
	}