﻿/*
  Script:	monitor.js
  Gadget:	Website & URL Monitor
  Purpose:	Monitor object defintion 'class' file
  Author:	Ben Coleman
  Update:	08 Oct 2007
  Notes:	Tested on Google Desktop v5.1
  Changes:	1.0.0.0	- Initial release
		1.1.0.0	- Added user & password to monitors and check for dodgy chars in monitor name
		1.1.1.0	- Monitors now sorted by name; and general code cleanup
		1.2.0.0	- Fixed some bugs, added last run time field, changed schedule from seconds to minutes
		1.5.0.0	- New font size options and update for Google Desktop v5.5

*/

// static values for status
STATUS_NA 		= -1;
STATUS_GOOD 	= 1;
STATUS_WARNING	= 2;
STATUS_ERROR 	= 3;

function Monitor(n, u, t, f, c, usr, pwd)
{
	// main values should be serialised
	this.name = n;
	this.url = u;
	this.threshold  = t;
	this.freq = f;
	this.check = c;
	this.user = usr;
	this.pass = pwd;

	// dynamic values used at runtime
	this.status = STATUS_NA;
	this.status_msg = "This monitor hasn't run yet";
	this.runcounter = 0;
	this.timer = 0;
	this.time = 0;
	this.status_prev = STATUS_NA;
	this.last_run = new Date();

	this.setStatus = function(stat, msg)
	{
		this.status_prev = this.status;
		this.status = stat;
		this.status_msg = msg;
	}

	this.run = function()
	{
		this.last_run = new Date();
		if(!framework.system.network.online && options.getValue("disble_when_offline")) {
			this.setStatus(STATUS_NA, "Monitoring is disabled due to no network connection "+framework.system.network.online);
			this.time = 0;
			updateStatus(this);		
			return;
		}
		this.status_msg = "Monitor is being run and updated";
		this.timer = new Date().getTime();
		this.http = new AJAX(createObjectCallback(this, this.runResponse), true, createObjectCallback(this, this.runError));
		if(this.user) {
			this.http.setAuth(this.user, this.pass);
		}
		rc = this.http.sendRequest(this.url);
		if(!rc) {
			this.setStatus(STATUS_ERROR, "Failed to send HTTP request to "+this.url);
			updateStatus(this);
		}
	}

	this.runResponse = function(body)
	{
		this.time = new Date().getTime() - this.timer;

		var temp_status = STATUS_GOOD;
		var temp_msg = "Monitor ran OK in "+this.time+" ms";

		if(this.time >= this.threshold) {
			temp_status = STATUS_WARNING;
			temp_msg = "Monitor took "+this.time+"ms, and exceeded the threshold";
		}
		
		if(this.check) {
			index = body.indexOf(this.check);
			if(index == -1) {
				temp_status = STATUS_ERROR;
				temp_msg = "Failed to find check text '"+this.check+"' on page";
			}
		}

		this.setStatus(temp_status, temp_msg);
		updateStatus(this);
	}

	this.runError = function(http)
	{
		err = http.statusText;
		if(err == "Unknown") {
			err = lookupHTTPErrorCode(http.status);
		}
		this.setStatus(STATUS_ERROR, "Monitor failed, "+err+" ("+http.status+")");
		updateStatus(this);
	}

	this.tick = function()
	{
		this.runcounter--;
		if(this.runcounter <= 0) {
			this.run();
			this.runcounter = this.freq;
		}
	}

	this.toListItem = function()
	{
		var f_size = options.getValue("font_size");
		var f_bold = options.getValue("font_bold");
		var y_off = 10 - f_size;
		return "<listitem cursor='hand' ondblclick='monitorDetails()' name='"+this.name+"'><img name='img' src='img/"+this.status+".png' /><label bold='"+f_bold+"' y='"+y_off+"' color='#ffffff' name='lab' x='16' font='Calibri' size='"+f_size+"'>"+this.name+"</label></listitem>";
	}

	this.statusText = function()
	{
		switch(this.status) {
			case STATUS_NA: 	return "Uninitalised";
			case STATUS_ERROR: 	return "Error";
			case STATUS_WARNING: 	return "Warning";
			case STATUS_GOOD: 	return "Good";
		}
	}
}

function monitorSort(mon_a, mon_b)
{
	if(mon_a.name < mon_b.name) {
		return -1;
	} else if(mon_a.name == mon_b.name) {
		return 0;
	} else if(mon_a.name > mon_b.name) {
		return 1;
	}
}
